import math
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd

np.set_printoptions(suppress=True)  # no E-notation

def linefit(x , y):
    N = float(len(x))
    sx,sy,sxx,syy,sxy=0,0,0,0,0
    for i in range(0,int(N)):
        sx  += x[i]
        sy  += y[i]
        sxx += x[i]*x[i]
        syy += y[i]*y[i]
        sxy += x[i]*y[i]
    a = (sy*sx/N -sxy)/( sx*sx/N -sxx)
    b = (sy - a*sx)/N
    r = abs(sy*sx/N-sxy)/math.sqrt((sxx-sx*sx/N)*(syy-sy*sy/N))
    return a,b,r
def read_data():
    path1 = './fit_data.csv'
    data= pd.read_csv(path1, header=None, names=['voltage', 'frequency', "concentrate","capacitance"])
    print(data.head())
    return data

def Line_fit(data):
    volt = [50, 100, 150, 200]
    freq = [20, 25, 30, 40, 50, 60, 70, 80, 90, 100, 200, 500, 800, 1000, 2000, 5000, 8000, 10000, 15000, 20000]

    bare_capa = 7.82701E-08
    list = [[0, 0, 0, 0, 0]]
    Fit_Result = np.array(list)
    for i in range(4):
        for j in range(20):
            volt_data = data.loc[data['voltage'] == volt[i]]
            volt_freq_data = volt_data.loc[volt_data["frequency"] == freq[j]]
            #print("(voltage,frequency)", (volt[i], freq[j]))
            #print(volt_freq_data)
            X = volt_freq_data["concentrate"].as_matrix()
            Y = volt_freq_data["capacitance"].as_matrix()
            init_capa = np.tile(7.82701E-08, (len(Y)))
            Y = 1e9 * (init_capa - Y)
            #print("X=", X)
            #print("Y=", Y)
            a, b, r = linefit(X, Y)
            result_list = [volt[i], freq[j], a, b, r]
            result_numpy = np.array([result_list])
            Fit_Result = np.r_[Fit_Result, result_numpy]
            # print("Fit Result: y = %10.5f x + %10.5f , r=%10.5f" % (a, b, r))

    Fit_Result = np.round(Fit_Result, decimals=4, out=None)
    Fit_Result_pd = pd.DataFrame(Fit_Result, columns=['Voltage', 'Frequency', "Slope", "Intercept", "R2"])
    return Fit_Result_pd

if __name__ == '__main__':
    data = read_data()
    Fit_Result_pd = Line_fit(data)
    Fit_Result_pd.to_csv('Fit_Result_pd.csv', header=True, index=False)
    print("Fit_Result\n", Fit_Result_pd)
    print(Fit_Result_pd[["a", "R2"]].idxmax(axis=0))



